@@ -54,7 +54,7 @@ def write(
         else:
             wSize += writer.write(data)
 
-        pbar.update(wSize)
+        pbar.update(blocklen)
 
     return wSize, cSizes
 
@@ -95,6 +95,6 @@ def read(
             nbdCon.nbd.pwrite(data, blockOffset)
             wSize += len(data)
 
-        pbar.update(wSize)
+        pbar.update(blocklen)
 
     return wSize
