@@ -14,6 +14,7 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
+
 import os
 import sys
 import glob
@@ -61,6 +62,15 @@ def printVersion(version) -> None:
     log.info("Version: %s Arguments: %s", version, " ".join(sys.argv))
 
 
+def humanize(num, suffix="B"):
+    """Print size in human readable output"""
+    for unit in ["", "K", "M", "G", "T", "P", "E", "Z"]:
+        if abs(num) < 1024.0:
+            return f"{num:3.1f}{unit}{suffix}"
+        num /= 1024.0
+    return f"{num:.1f}Yi{suffix}"
+
+
 def setThreadName(tn="main") -> None:
     """Set thread name reported by logging function"""
     current_thread().name = tn
@@ -80,7 +90,7 @@ def sshSession(
 ) -> Union[ssh.client, None]:
     """Use ssh to copy remote files"""
     try:
-        return ssh.client(remoteHost, args.ssh_user, mode)
+        return ssh.client(remoteHost, args.ssh_user, args.ssh_port, mode)
     except sshError as err:
         log.warning("Failed to setup SSH connection: [%s]", err)
 
@@ -224,6 +234,7 @@ def progressBar(total: int, desc: str, args: Namespace, count=0) -> tqdm:
         desc=desc,
         unit="B",
         unit_scale=True,
+        unit_divisor=1000,
         disable=args.noprogress,
         position=count,
         leave=False,
