@@ -18,7 +18,6 @@ import os
 import string
 import random
 import logging
-from socket import gethostname
 from argparse import Namespace
 from typing import Any, Dict, List, Tuple, Union
 import libvirt
@@ -58,16 +57,18 @@ class client:
 
     @staticmethod
     def _connectAuth(uri: str, user: str, password: str) -> libvirt.virConnect:
-        """Use openAuth if connection string includes authfile or
-        username/password are set"""
+        """Use openAuth if connection for advanced SASL authentication mechanisms
+        if username and password are set"""
 
-        def _cred(credentials, user_data) -> None:
+        def _cred(credentials, user_data) -> int:
             for credential in credentials:
                 if credential[0] == libvirt.VIR_CRED_AUTHNAME:
                     credential[4] = user_data[0]
                 elif credential[0] == libvirt.VIR_CRED_PASSPHRASE:
                     credential[4] = user_data[1]
 
+            return 0
+
         log.debug("Username: %s", user)
         log.debug("Password: %s", password)
 
@@ -85,73 +86,41 @@ class client:
 
     @staticmethod
     def _connectOpen(uri: str) -> libvirt.virConnect:
-        """Open connection with regular libvirt URI for local authentication"""
+        """Open connection with regular libvirt URI for local authentication
+        without further authentication mechanisms required"""
         try:
             return libvirt.open(uri)
         except libvirt.libvirtError as e:
-            raise connectionFailed(e) from e
-
-    @staticmethod
-    def _reqAuth(uri: str) -> bool:
-        """If authentication file is passed or qemu+ssh is used,
-        no user and password are required."""
-        return "authfile" in uri
-
-    @staticmethod
-    def _isSsh(uri: str) -> bool:
-        """If authentication file is passed or qemu+ssh is used,
-        no user and password are required."""
-        return uri.startswith("qemu+ssh")
-
-    def _useAuth(self, args: Namespace) -> bool:
-        """Check if we want to use advanced auth method"""
-        if args.uri.startswith("qemu+"):
-            return True
-        if self._reqAuth(args.uri):
-            return True
-        if args.user or args.password:
-            return True
-
-        return False
+            if e.get_error_code() == 45:
+                errmsg = f"{e}: --user and --password options for SASL authentication are required."
+                raise connectionFailed(errmsg) from e
+        return None
 
     def _connect(self, args: Namespace) -> libvirt.virConnect:
-        """return libvirt connection handle"""
+        """return libvirt connection handle and check if connection
+        is established to a remote host."""
         log.debug("Libvirt URI: [%s]", args.uri)
-        localHostname = gethostname()
-        log.debug("Hostname: [%s]", localHostname)
 
-        if self._useAuth(args):
-            log.debug(
-                "Login information specified, connect libvirtd using openAuth function."
-            )
-            if (
-                not self._reqAuth(args.uri)
-                and not self._isSsh(args.uri)
-                and (not args.user or not args.password)
-            ):
-                raise connectionFailed(
-                    "Username (--user) and password (--password) required."
-                )
-            if not self._isSsh(args.uri):
-                conn = self._connectAuth(args.uri, args.user, args.password)
-            else:
-                conn = self._connectOpen(args.uri)
+        if args.user and args.password:
+            conn = self._connectAuth(args.uri, args.user, args.password)
+        else:
+            conn = self._connectOpen(args.uri)
 
+        # Detect if we are connected to a remote libvirt daemon by
+        # comparing the local and remote hostname. If qemu+ssh is
+        # part of the libvirt URI, set the remote host as well.
+        # This will spawn the NBD service for data transfer via
+        # TCP socket instead of local socket file and related virtual
+        # domain files will be copied via SFTP.
+        if "qemu+ssh" in args.uri:
             remoteHostname = conn.getHostname()
-            log.debug("Hostname returned by libvirt API: [%s]", remoteHostname)
-            if localHostname != remoteHostname:
-                log.info(
-                    "Connected to remote host: [%s], local host: [%s]",
-                    conn.getHostname(),
-                    gethostname(),
-                )
-                self.remoteHost = remoteHostname
+            log.info(
+                "Connected to remote host: [%s]",
+                remoteHostname,
+            )
+            self.remoteHost = remoteHostname
 
-            return conn
-
-        log.debug("Connect libvirt using open function.")
-
-        return self._connectOpen(args.uri)
+        return conn
 
     def close(self) -> None:
         """Disconnect"""
@@ -226,6 +195,13 @@ class client:
         """Return Virtual Machine configuration as XML"""
         return domObj.XMLDesc(0)
 
+    @staticmethod
+    def startDomain(domObj: libvirt.virDomain) -> bool:
+        """Start virtual machine in paused state to allow full / inc backup"""
+        return domObj.createWithFlags(
+            flags=libvirt.VIR_DOMAIN_START_PAUSED | libvirt.VIR_DOMAIN_START_AUTODESTROY
+        )
+
     @staticmethod
     def domainAutoStart(domObj: libvirt.virDomain) -> None:
         """Mark virtual machine for autostart"""
@@ -265,6 +241,16 @@ class client:
         log.debug("Domain Info: [%s]", settings)
         return settings
 
+    def getTPMDevice(self, vmConfig: str) -> bool:
+        """Check if virtual machine has configured an emulated (swtpm based) TPM device"""
+        tree = xml.asTree(vmConfig)
+        device = tree.find("devices/tpm")
+        if device is not None:
+            tpm = device.xpath("backend")[0].get("type")
+            return tpm == "emulator"
+
+        return False
+
     @staticmethod
     def getBackingStores(disk: xml._Element) -> List[str]:
         """Get list of backing store files defined for disk, usually
